package de.fhw.swtvertiefung.knauf.security.userregistry;

import java.rmi.RemoteException;
import java.security.cert.X509Certificate;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import com.ibm.websphere.security.CertificateMapFailedException;
import com.ibm.websphere.security.CertificateMapNotSupportedException;
import com.ibm.websphere.security.CustomRegistryException;
import com.ibm.websphere.security.EntryNotFoundException;
import com.ibm.websphere.security.NotImplementedException;
import com.ibm.websphere.security.PasswordCheckFailedException;
import com.ibm.websphere.security.Result;
import com.ibm.websphere.security.UserRegistry;
import com.ibm.websphere.security.cred.WSCredential;

/**
 * UserRegistry-Implementation fr dieses Projekt.
 */
public class KnaufUserRegistry implements UserRegistry
{
  private String DBDRIVER = null;

  private String DBURL = null;

  private String DBUSERNAME = null;

  private String DBPASSWORD = null;

  private String DBSCHEMA = null;

  private Connection con = null;

  private Statement stmt = null;

  /** Die Rolle "Administrator". */
  private String roleAdmin = "administrator";

  /** Group-ID des Admins */
  private String groupIdAdmin = "GIDadmin";

  /** Die Rolle "Student". */
  private String roleStudent = "student";

  /** Group-ID des Studenten */
  private String groupIdStudent = "GIDstudi";

  /**
   * Konstruktor.
   *  
   */
  public KnaufUserRegistry()
  {
    System.err.println("KnaufUserRegistry->Konstruktor !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#initialize(java.util.Properties)
   */
  public void initialize(Properties props) throws CustomRegistryException, RemoteException
  {
    System.err.println("initialize: " + props);
    //Verbindung herstellen:
    this.DBDRIVER = props.getProperty("DBDRIVER");
    this.DBURL = props.getProperty("DBURL");
    this.DBPASSWORD = props.getProperty("DBPASSWORT");
    this.DBUSERNAME = props.getProperty("DBUSER");
    this.DBSCHEMA = props.getProperty("DBSCHEMA");
    //Schemaangabe NULL in Leerstring umwandeln, sonst pat
    // String-Concatenation nicht.
    if (this.DBSCHEMA == null)
      this.DBSCHEMA = "";
    //Wenn Schema angegeben ist (kein Leerstring), dann muss es mit Punkt enden.
    //Wenn nicht, zufgen.
    if (this.DBSCHEMA.length() > 0 && this.DBSCHEMA.endsWith(".") == false)
      this.DBSCHEMA += ".";
    try
    {
      // connect to database...
      Driver driver = (Driver) Class.forName(DBDRIVER).newInstance();
      System.err.println("driver: " + driver);
      DriverManager.registerDriver(driver);
      con = DriverManager.getConnection(DBURL, DBUSERNAME, DBPASSWORD);
      stmt = con.createStatement();
      System.err.println("initialize: Datenbankverbindung steht");
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.err.println("initialize: Datenbankverbindung schlug fehl");
      //throw new CustomRegistryException(e.getMessage());
    }
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#checkPassword(java.lang.String,
   *      java.lang.String)
   */
  public String checkPassword(String userSecurityName, String password) throws PasswordCheckFailedException,
      CustomRegistryException, RemoteException
  {
    System.err.println("checkPassword mit userSecurityName: " + userSecurityName + ", password: " + password);
    try
    {
      //Student laden ?
      Student student = this.loadStudentByLogin(userSecurityName);
      if (student != null)
      {
        //Student mit Login wurde gefunden:
        if (password.equals(student.getPasswort().trim()))
        {
          System.err.println("checkPassword succeeded for " + userSecurityName);
          return userSecurityName;
        }
        else
          throw new PasswordCheckFailedException("Student-Passwort fr Login " + userSecurityName + " falsch");
      }
      else
      {
        //Ist es ein Admin ?
        Administrator admin = this.loadAdminByLogin(userSecurityName);
        if (admin != null)
        {
          //Admin mit Login wurde gefunden:
          if (password.equals(admin.getPasswort().trim()))
          {
            System.err.println("checkPassword succeeded for " + userSecurityName);
            return userSecurityName;
          }
          else
            throw new PasswordCheckFailedException("Administrator-Passwort fr Login " + userSecurityName + " falsch");
        }
        else
        {
          //Weder Admin noch Student fr Login gefunden.
          throw new PasswordCheckFailedException("Weder Student noch Admin fr Login '" + userSecurityName
              + " gefunden.");
        }
      }
    }
    catch (SQLException e)
    {
      throw new CustomRegistryException(e.getMessage());
    }
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#mapCertificate(java.security.cert.X509Certificate[])
   */
  public String mapCertificate(X509Certificate[] cert) throws CertificateMapNotSupportedException,
      CertificateMapFailedException, CustomRegistryException, RemoteException
  {
    System.err.println("mapCertificate: " + cert);
    if (cert == null)
      return null; // special case
    String name = null;
    X509Certificate cert1 = null;
    if (cert.length > 0)
      cert1 = cert[0];
    else
      throw new CertificateMapFailedException("Certificate not found");
    try
    {
      // map the SubjectDN in the certificate to a userID.
      name = cert1.getSubjectDN().getName();
    }
    catch (Exception e)
    {
      throw new CertificateMapNotSupportedException(e.getMessage());
    }
    try
    {
      this.getUniqueUserId(name);
    }
    catch (EntryNotFoundException e)
    {
      throw new CertificateMapFailedException(e.getMessage());
    }
    return name;
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getRealm()
   */
  public String getRealm() throws CustomRegistryException, RemoteException
  {
    System.err.println("getRealm");
    return null;
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUsers(java.lang.String,
   *      int)
   */
  public Result getUsers(String pattern, int limit) throws CustomRegistryException, RemoteException
  {
    System.err.println("<not implemented>getUsers: pattern = " + pattern + ", Limit = " + limit);

    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUserDisplayName(java.lang.String)
   */
  public String getUserDisplayName(String userSecurityName) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    System.err.println("<not implemented>getUserDisplayName: " + userSecurityName);
    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUniqueUserId(java.lang.String)
   */
  public String getUniqueUserId(String userSecurityName) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    //Unique UserID des Studenten oder Admins laden:
    System.err.println("getUniqueUserId for Username " + userSecurityName);
    try
    {
      //Student laden ?
      Student student = this.loadStudentByLogin(userSecurityName);
      if (student != null)
      {
        //Student mit Login wurde gefunden:
        //ID.toString zurckgeben.
        System.err
            .println("getUniqueUserId for Username " + userSecurityName + " -> Student mit ID " + student.getId());
        return student.getId().toString();
      }
      else
      {
        //Ist es ein Admin ?
        Administrator admin = this.loadAdminByLogin(userSecurityName);
        if (admin != null)
        {
          //Admin mit Login wurde gefunden:
          //ID.toString zurckgeben.
          System.err.println("getUniqueUserId for Username " + userSecurityName + " -> Admin mit ID " + admin.getId());
          return admin.getId().toString();
        }
        else
        {
          //Weder Admin noch Student fr Login gefunden.
          throw new EntryNotFoundException("Weder Student noch Admin fr Login '" + userSecurityName + " gefunden.");
        }
      }
    }
    catch (SQLException e)
    {
      throw new CustomRegistryException(e.getMessage());
    }
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUserSecurityName(java.lang.String)
   */
  public String getUserSecurityName(String uniqueUserId) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    System.err.println("<not implemented>getUserSecurityName: " + uniqueUserId);

    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#isValidUser(java.lang.String)
   */
  public boolean isValidUser(String userSecurityName) throws CustomRegistryException, RemoteException
  {
    System.err.println("<not implemented>isValidUser: " + userSecurityName);

    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getGroups(java.lang.String,
   *      int)
   */
  public Result getGroups(String pattern, int limit) throws CustomRegistryException, RemoteException
  {
    System.err.println("<not implemented>getGroups: pattern = " + pattern + ", Limit = " + limit);
    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getGroupDisplayName(java.lang.String)
   */
  public String getGroupDisplayName(String groupSecurityName) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    System.err.println("<not implemented>getGroupDisplayName: " + groupSecurityName);
    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUniqueGroupId(java.lang.String)
   */
  public String getUniqueGroupId(String groupSecurityName) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    System.err.println("getUniqueGroupId: " + groupSecurityName);

    if (groupSecurityName.equals(this.roleAdmin))
    {
      System.err.println("Admin (ID = " + this.groupIdAdmin + ")");
      return this.groupIdAdmin;
    }
    else if (groupSecurityName.equals(this.roleStudent))
    {
      System.err.println("Student (ID = " + this.groupIdStudent + ")");
      return this.groupIdStudent;
    }
    else
    {
      System.err.println("ID not found " + groupSecurityName);
      throw new EntryNotFoundException("Ungltiger Gruppenname: " + groupSecurityName);
    }
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUniqueGroupIds(java.lang.String)
   */
  public List getUniqueGroupIds(String uniqueUserId) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    //Gruppen des Users:
    ArrayList listGruppen = new ArrayList();
    //Unique UserID des Studenten oder Admins laden:
    System.err.println("getUniqueGroupIds for Userid " + uniqueUserId);
    try
    {
      //Student laden ?
      Student student = this.loadStudentById(uniqueUserId);
      if (student != null)
      {
        //Student mit Login wurde gefunden:
        //ID.toString zurckgeben.
        System.err.println("getUniqueGroupIds for Userid " + uniqueUserId + " -> Student gefunden");
        listGruppen.add(this.groupIdStudent);
      }
      else
      {
        //Ist es ein Admin ?
        Administrator admin = this.loadAdminById(uniqueUserId);
        if (admin != null)
        {
          //Admin mit Login wurde gefunden:
          //ID.toString zurckgeben.
          System.err.println("getUniqueGroupIds for Userid " + uniqueUserId + " -> Admin gefunden");
          listGruppen.add(this.groupIdAdmin);
        }
        else
        {
          //Weder Admin noch Student fr Login gefunden.
          throw new EntryNotFoundException("Weder Student noch Admin fr ID '" + uniqueUserId + " gefunden.");
        }
      }
    }
    catch (SQLException e)
    {
      throw new CustomRegistryException(e.getMessage());
    }

    return listGruppen;
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getGroupSecurityName(java.lang.String)
   */
  public String getGroupSecurityName(String uniqueGroupId) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    System.err.println("<not implemented>getGroupSecurityName: " + uniqueGroupId);

    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#isValidGroup(java.lang.String)
   */
  public boolean isValidGroup(String groupSecurityName) throws CustomRegistryException, RemoteException
  {
    System.err.println("<not implemented>isValidGroup: " + groupSecurityName);

    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getGroupsForUser(java.lang.String)
   */
  public List getGroupsForUser(String userSecurityName) throws EntryNotFoundException, CustomRegistryException,
      RemoteException
  {
    System.err.println("<not implemented>getGroupsForUser: " + userSecurityName);

    throw new CustomRegistryException("Not implemented !");
  }

  /**
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#getUsersForGroup(java.lang.String,
   *      int)
   * @deprecated
   */
  public Result getUsersForGroup(String groupSecurityName, int limit) throws NotImplementedException,
      EntryNotFoundException, CustomRegistryException, RemoteException
  {
    System.err.println("<not implemented>getUsersForGroup: " + groupSecurityName);

    throw new CustomRegistryException("Not implemented !");
  }

  /*
   * (Kein Javadoc)
   * 
   * @see com.ibm.websphere.security.UserRegistry#createCredential(java.lang.String)
   */
  public WSCredential createCredential(String userSecurityName) throws NotImplementedException, EntryNotFoundException,
      CustomRegistryException, RemoteException
  {
    System.err.println("<not implemented>createCredential: " + userSecurityName);
    throw new NotImplementedException();
  }

  ////////////////////////////////////////////////////////////////////////
  //                 Hilfsmethoden //
  ////////////////////////////////////////////////////////////////////////
  /**
   * Laden eines Studenten anhand des Logins.
   * 
   * @return Den Studenten oder NULL, wenn nicht gefunden.
   */
  private Student loadStudentByLogin(String str_LoginName) throws SQLException
  {
    ResultSet rs = stmt.executeQuery("SELECT ID, NAME, LOGIN, PASSWORT FROM " + DBSCHEMA + "STUDENT WHERE LOGIN='"
        + str_LoginName + "'");
    Student student = null;
    if (rs.next())
    {
      student = new Student();
      student.fromResultSet(rs);
    }
    rs.close();
    return student;
  }

  /**
   * Laden eines Administrators anhand des Logins.
   * 
   * @return Den Administrator oder NULL, wenn nicht gefunden.
   */
  private Administrator loadAdminByLogin(String str_LoginName) throws SQLException
  {
    ResultSet rs = stmt.executeQuery("SELECT ID, NAME, LOGIN, PASSWORT FROM " + DBSCHEMA
        + "ADMINISTRATOR WHERE LOGIN='" + str_LoginName + "'");
    Administrator admin = null;
    if (rs.next())
    {
      admin = new Administrator();
      admin.fromResultSet(rs);
    }
    rs.close();
    return admin;
  }

  /**
   * Laden eines Studenten anhand der Id.
   * 
   * @param str_Id
   *          Die ID
   * @return Den Studenten oder NULL, wenn nicht gefunden.
   */
  private Student loadStudentById(String str_Id) throws SQLException
  {
    Integer intID = Integer.valueOf(str_Id);
    ResultSet rs = stmt.executeQuery("SELECT ID, NAME, LOGIN, PASSWORT FROM " + DBSCHEMA + "STUDENT WHERE ID="
        + intID);
    Student student = null;
    if (rs.next())
    {
      student = new Student();
      student.fromResultSet(rs);
    }
    rs.close();
    return student;
  }

  /**
   * Laden eines Administrators anhand des Logins.
   * 
   * @param str_Id
   *          Die ID
   * @return Den Administrator oder NULL, wenn nicht gefunden.
   */
  private Administrator loadAdminById(String str_Id) throws SQLException
  {
    Integer intID = Integer.valueOf(str_Id);
    ResultSet rs = stmt.executeQuery("SELECT ID, NAME, LOGIN, PASSWORT FROM " + DBSCHEMA + "ADMINISTRATOR WHERE ID="
        + intID);
    Administrator admin = null;
    if (rs.next())
    {
      admin = new Administrator();
      admin.fromResultSet(rs);
    }
    rs.close();
    return admin;
  }
}
